from client.TradingInterface.TradingInterface import TradingInterface
import requests
import json

BASE_URL = "https://14.97.57.98:44551/api"
ROUTE_LOGIN = "/accounts/login/"
ROUTE_ORDER = "/orders/"
ROUTE_MARGIN = ROUTE_ORDER + "margin/"
ROUTE_POSITIONS = ROUTE_ORDER + "positions/"
ROUTE_TRADES = ROUTE_ORDER + "trades/"
ROUTE_HOLDINGS = ROUTE_ORDER + "holdings/"
ROUTE_MPIN = "/accounts/mpin/"


def refreshAccessToken(refresh_token) -> str:
    return "access_token"


def getSymbolId(trading_symbol):
    # TODO: Refresh SYMBOL IDs cron day - 7AM

    pass


class UTradeBroker(TradingInterface):
    def __init__(self, username, password, baseUrl=BASE_URL):
        self.username = username
        self.password = password
        self.token = ""
        self.base_url = baseUrl
        self.login_token = ""
        self.access_token = ""
        self.refresh_token = ""

    def Login(self) -> str:
        payload = json.dumps({
            "username": self.username,
            "password": self.password,
            "platform": "web"
        })
        headers = {
            'Content-Type': 'application/json',
            'App-Secret': '1'
        }
        response = requests.request("POST", self.base_url + ROUTE_LOGIN, headers=headers, data=payload, verify=False)
        if response.status_code == 200:
            res = response.json()
            print(res)
            self.login_token = res['two_factor_token']
            return "logged_in"
        return "login_failed"

    def PlaceOrder(self, qty, transaction_type: str, trading_symbol, exchange, product_type):
        #         TODO: Add refresh Access Token Code
        # smy_id = getSymbolId(trading_symbol)
        # self.access_token = refreshAccessToken(self.refresh_token)
        payload = json.dumps({
            "side": transaction_type.lower(),
            "product_type": "Delivery",
            "symbol_id": trading_symbol,
            "order_type": "market",
            "quantity": qty,
            "disclosed_qty": 0,
            "disclosed": "0",
            "square_off": 0,
            "validity": "day"
        })
        headers = {
            'Authorization': 'Token ' + self.access_token,
            'Content-Type': 'application/json',
            'App-Secret': '1'
        }

        response = requests.request("POST", self.base_url + ROUTE_ORDER, headers=headers, data=payload, verify=False)
        if response.status_code == 200:
            print("Request Data->")
            print(payload)
            print(response.json())
        else:
            print("Order Not Placed->", response.json())

    def Margins(self):
        headers = {
            'Authorization': 'Token ' + self.access_token,
            'Content-Type': 'application/json',
            'App-Secret': '1'
        }
        print("Url->{}".format(self.base_url + ROUTE_MARGIN))
        print("headers->{}".format(headers))
        response = requests.request("GET", self.base_url + ROUTE_MARGIN, headers=headers, verify=False)
        if response.status_code == 200:
            print(response.json())
            data = {
                "net_bal": response.json()['net_mtm'],
                "cash_available": response.json()['cash_available']
            }
        else:
            print("Error in Fetching margin->", response.json())
            data = {
                "net_bal": 0,
                "cash_available": 0
            }
        return data

    def Positions(self):
        headers = {
            'Authorization': 'Token ' + self.access_token,
            'Content-Type': 'application/json',
            'App-Secret': '1'
        }
        print("Url-->", self.base_url + ROUTE_POSITIONS)
        print("Headers->", headers)
        response = requests.request("GET", self.base_url + ROUTE_POSITIONS, headers=headers, verify=False)
        print("I am here")
        position = list()
        if response.status_code == 200:
            # print(response.json())
            # TODO: Preprocessing Required
            for pos in response.json()['data']:
                position.append({'symbol_id': pos['symbol_id'],
                                 'tradingsymbol': pos['symbol_id'],
                                 'quantity': pos['buy_qty'] - pos['sell_qty'],
                                 'pnl': pos['net_profit_loss'],
                                 'buy_quantity': pos['buy_qty'],
                                 'sell_quantity': pos['sell_qty']})
            return position
        else:
            print("Error in Fetching margin->", response.json())
            return []

    def GenerateSession(self, mPin=123456) -> bool:
        payload = json.dumps({
            "pin": mPin,
            "token": self.login_token,
            "username": self.username,
            "platform": "web"
        })
        headers = {
            'Content-Type': 'application/json',
        }
        response = requests.request("POST", self.base_url + ROUTE_MPIN, headers=headers, data=payload, verify=False)
        if response.status_code == 200:
            res = response.json()
            print(res['access_token'])
            self.access_token = res['access_token']
            self.refresh_token = res['refresh_token']
            return True
        return False

    def GetInstruments(self):
        super().GetInstruments()

    def Holdings(self):
        headers = {
            'Authorization': 'Token ' + self.access_token,
            'Content-Type': 'application/json',
            'App-Secret': '1'
        }
        print("Url->", self.base_url + ROUTE_HOLDINGS)
        print("Headers->", headers)
        response = requests.request("GET", self.base_url + ROUTE_HOLDINGS, headers=headers, verify=False)
        if response.status_code == 200:
            # print(response.json())
            # TODO: Preprocessing Required
            return response.json()
        else:
            print("Error in Fetching margin->", response.json())
            return []

    def GetQuote(self, listInstruments, exchange):
        super().GetQuote(listInstruments, exchange)

    def Orders(self):
        super().Orders()

    def Profile(self):
        super().Profile()

    def GetAvailableMargin(self):
        headers = {
            'Authorization': 'Token ' + self.access_token,
            'Content-Type': 'application/json',
            'App-Secret': '1'
        }
        print("Url->{}".format(self.base_url + ROUTE_MARGIN))
        print("headers->{}".format(headers))
        response = requests.request("GET", self.base_url + ROUTE_MARGIN, headers=headers, verify=False)
        if response.status_code == 200:
            print(response.json())
            data = {'adhoc_margin': response.json()['adhoc'],
                    'cash': response.json()['cash'],
                    'opening_balance': response.json()['cash'],
                    'live_balance': response.json()['cash'],
                    'collateral': response.json()['collateral'],
                    'intraday_payin': response.json()['pay_in']}
            return data
        else:
            return {'adhoc_margin': 0,
                    'cash': 0,
                    'opening_balance': 0,
                    'live_balance': 0,
                    'collateral': 0,
                    'intraday_payin': 0}



from kiteconnect import KiteConnect, exceptions
from rest_framework import status
from client.TradingInterface.TradingInterface import TradingInterface


def modPositions(positions):
    req_col = ['tradingsymbol', 'pnl', 'quantity', 'buy_quantity', 'sell_quantity']
    mod_positions = list()
    mod_positions.append(req_col)
    for position in positions:
        pos = {}
        for key, value in position.items():
            if key in req_col:
                pos[key] = value
        mod_positions.append(pos)
    return mod_positions


class KiteTrading(TradingInterface):
    def __init__(self, api_key: str, api_secret: str, request_token: str, access_token: str,
                 is_session_generated: bool, truinvest_secret: str):
        self.api_key = api_key
        self.api_secret = api_secret
        self.request_token = request_token
        self.access_token = access_token
        self.kite = KiteConnect(api_key=self.api_key)
        if is_session_generated:
            self.kite.set_access_token(self.access_token)

    def Login(self):
        return self.kite.login_url()

    def GenerateSession(self, mPin: str):
        data = {}
        # user_data
        try:
            data['status'] = status.HTTP_201_CREATED
            user_data = self.kite.generate_session(self.request_token, api_secret=self.api_secret)
            print(user_data)
            data['user'] = user_data
            self.kite.set_access_token(user_data['access_token'])
            self.access_token = user_data['access_token']
        except exceptions.TokenException as exp:
            print('Error-{}'.format(exp))
            data['status'] = status.HTTP_403_FORBIDDEN
            # data['user'] = self.connectKite()
            # data['error'] = exp
        return data

    def PlaceOrder(self, qty, transaction_type, trading_symbol, exchange, product_type):
        try:
            print("order {}-{}-{}".format(trading_symbol, qty, transaction_type, self.kite.PRODUCT_CNC))
            # entryTrade = 0
            entryTrade = self.kite.place_order(tradingsymbol=trading_symbol,
                                               exchange=exchange,
                                               transaction_type=transaction_type,
                                               quantity=qty,
                                               order_type=self.kite.ORDER_TYPE_MARKET,
                                               product=product_type,
                                               variety=self.kite.VARIETY_REGULAR
                                               )
        except Exception as E:
            print("Order Not Placed")
            print("trading_symbol-{}".format(trading_symbol))
            print("transaction_type-{}".format(transaction_type))
            print("Qty-{}".format(qty))
            print("Error-{}".format(E))
            entryTrade = 0
        return entryTrade

    def Margins(self):
        data = dict()
        try:
            data['status'] = status.HTTP_200_OK
            margins = self.kite.margins()
            print(margins)
            net_bal = margins['equity']['net']
            cash_bal = margins['equity']['available']['cash']
            data['user'] = (net_bal, cash_bal)
        except Exception as e:
            data['status'] = status.HTTP_401_UNAUTHORIZED
            print("Error-{}".format(e))
            # data['user'] = self.kite.connectKite()
        return data

    def Positions(self):
        pos_net = modPositions(self.kite.positions()['net'])
        pos_day = modPositions(self.kite.positions()['day'])
        return pos_net, pos_day

    def GetInstruments(self):
        return self.kite.instruments()

    def GetHoldings(self):
        data = {
            'holdings': self.kite.holdings(),
            'mf_holdings': self.kite.mf_holdings()
        }
        return data

    def GetQuote(self, listInstruments, exchange):
        if exchange == "NSE":
            for i in range(len(listInstruments)):
                listInstruments[i] = 'NSE:' + listInstruments[i]
        return self.kite.quote(listInstruments)

    def Orders(self):
        return self.kite.orders()

    def Profile(self):
        return self.kite.profile()

    def GetAvailableMargin(self):
        return self.kite.margins()['equity']['available']

    def GetUtilizedMargin(self):
        return self.kite.margins()['equity']['Utilized']



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="truinvest-broker",
    version="1.0.7",
    author="Anukkrit Shanker",
    author_email="anukkrit@rebal.tech",
    description="Broker client Lib",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://truinvest.ai/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.9',
    install_requires=[
        'gitpython',
        'kiteconnect',
        'djangorestframework',
        'requests',
        'pandas'
    ],
    keywords='truinvest truinvest-broker',
    project_urls={
        'Homepage': 'https://truinvest.ai/',
    },
)